/* Based on code from http://www.rcgroups.com/forums/showthread.php?t=1817699
   used libraryes:
     https://github.com/adafruit/Adafruit-GFX-Library
     https://github.com/adafruit/Adafruit_SSD1306
     https://github.com/NiqueIt/TeensyRC 
     https://github.com/sgofferj/arduino-frsky 
*/

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <EEPROM.h>
#include <PulsePosition.h>

#define OLED_RESET 4
#define FRSKY_TELEMETRY
#define THROTLE_CUT 

#if 1
__asm volatile ("nop");
#endif

#ifdef FRSKY_TELEMETRY
  #include <FrSky.h>
  #define frskTelemRx Serial1
  FrSky frsky;
  int frskyRxCnt = 0;
#endif 

#define PCF8574_TRIM_ADDR 0x20

Adafruit_SSD1306 display(OLED_RESET);
PulsePositionOutput ppmOut;

// config PPM
#define servoCentro 1500       //servo center value (us)
#define ppmLong 22500          //ppm frame
#define ppmPulso 300           //ppm pulse width
#define onState 0              //Polarity: 0 neg / 1 pos
#define sigPin 6               //ppm output pin
#define ppmMin 1000
#define ppmMax 2000

// config TX
#define CANALES 8              //number of channels
#define MODELOS 5              //total model memory
#define canalTh 2              //Throtle channel ID
#define potMin 0               //Pot min value
#define potMax 1023            //Por max value
#define txVer "TeensyTX v1.0.2"
#define swTC 8
#define swDR 7
#define tonePin 5

#define SUBTRIM_MIN -256
#define SUBTRIM_MAX 256

#define MODEL_EEPROM_OFFSET 511
#define CALIBR_EEPROM_OFFSET 1024
#define numBytesPorModelo 64

int ppm[CANALES];              //ppm output array
unsigned char sw_status=0x00;  //0:TC 1:DR 
unsigned int pots[CANALES];    //ADC data array
unsigned char inPots[CANALES] = {
  A0, A9, A1, A8, 255, 255, 255, 255
}; //Input pins array
unsigned char batPin = A6;
int saveSubtrim = 0;

// nothing editable from here :)
// *****************************
unsigned char menuActual=0;
unsigned char menuSubActual=0;
unsigned char submenuActual=0;
unsigned char drSeleccion=0xFF;
unsigned char pantalla=0;
unsigned char menu=0;
unsigned char modeloActual=0;
unsigned char servoReversa;
unsigned char dual_rate_low[CANALES];
unsigned char dual_rate_hi[CANALES];
unsigned char exponent[CANALES];
short sub_trim[CANALES];
boolean displayReady=false;
int temporal=0;
int calibracion[CANALES][2];
char msgs[][19] = {
  "Main Menu",
  "Servo Direction", 
  "Dual Rates/Expon", 
  "Model Selection", 
  "Save data", 
  "Stick Calibration"
};

#if defined(__AVR__)
// timer1 setup
void configTimer1() {
  cli();
  TCCR1A = 0;
  TCCR1B = 0;
  OCR1A = 100;
  TCCR1B |= (1 << WGM12);
  TCCR1B |= (1 << CS11);  // prescaler 8: 0.5us - 16mhz
  TIMSK1 |= (1 << OCIE1A);
  sei();
}

// Interrupcion del timer
ISR(TIMER1_COMPA_vect){  
  static boolean state = true;
  TCNT1 = 0;
  if(state) {   
    digitalWrite(sigPin, onState);
    OCR1A = ppmPulso * 2;
    state = false;
  }
  else{   
    static byte cur_chan_numb;
    static unsigned int calc_rest;
    digitalWrite(sigPin, !onState);
    state = true;
    if(cur_chan_numb >= CANALES){
      cur_chan_numb = 0;
      calc_rest = calc_rest + ppmPulso; 
      OCR1A = (ppmLong - calc_rest) * 2;
      calc_rest = 0;
    }
    else{
      OCR1A = (ppm[cur_chan_numb] - ppmPulso) * 2;
      calc_rest = calc_rest + ppm[cur_chan_numb];
      cur_chan_numb++;
    }     
  }
}
#endif // __AVR__

void setup() {
  unsigned char i;

  // switches setuo
#ifdef THROTLE_CUT 
  pinMode(swTC, INPUT_PULLUP);
  digitalWrite(swTC, HIGH); // pullup
#endif
  pinMode(swDR, INPUT_PULLUP);
  digitalWrite(swDR, HIGH); // pullup
  pinMode(2, INPUT_PULLUP);
  pinMode(3, INPUT_PULLUP);
  pinMode(4, INPUT_PULLUP);
  digitalWrite(2, HIGH); // pullup
  digitalWrite(3, HIGH); // pullup
  digitalWrite(4, HIGH); // pullup

  // zero state values for ppm
  ppm[0]=servoCentro;
  ppm[1]=servoCentro;
  ppm[2]=ppmMin;
  ppm[3]=servoCentro;
  ppm[4]=servoCentro;
  ppm[5]=servoCentro;
  ppm[6]=servoCentro;
  ppm[7]=servoCentro;

  //zero state config parameters

  // servo reverse: 0 normal, 1 reverse
  servoReversa=0b00000000;

  // dual rates por defecto
  for (i=0; i < CANALES; i++) {
    if(i != canalTh)
      dual_rate_low[i]=75;
    else  
      dual_rate_low[i]=100;
    dual_rate_hi[i]=100;
    exponent[i]=30                                                                                                                                                                                        ;
    sub_trim[i] = 0;
  }

  // load data from EEPROM
  //reseteaEEPROM();
  modeloActual=cargaDatosEEPROM(204);

  // LCD config
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(10,0);
  display.println(txVer);
  display.setCursor(10,17);  
  display.println("(c) 2014");
  display.display();
  delay(1000);
  
  // check throtle 
  readSwitches();
  readPots();
  if(pots[canalTh] > 5 )
  {
    display.clearDisplay();
    display.setCursor(5,17); 
    display.println("Throtle not idle");
    display.display();
    tone(tonePin, 500);
    delay(200);
    tone(tonePin, 1500);
    delay(200);
    tone(tonePin, 500);
    delay(200);
    noTone(tonePin);
    while(readKeys() == 0);
  }
  
  // ppm timer config
#if defined(__AVR__)
  pinMode(sigPin, OUTPUT);
  digitalWrite(sigPin, !onState);
  configTimer1();
#else
  pinMode(sigPin, OUTPUT);
  ppmOut.begin(sigPin);
#endif
  pantalla = 0;

#ifdef FRSKY_TELEMETRY
  frskTelemRx.begin(9600);
#endif

}


void loop() {
  int k;
  int i;
  int j=0;
  unsigned char tecla=0;
  short int vBat;
  long vBatTmp = 0;
  unsigned char valT;
  vBat=analogRead(batPin);
  while(1) {
    k++;
    
    vBatTmp+=analogRead(batPin);
    if((k%50) == 0)
    {
      vBat = vBatTmp /50;
      vBatTmp = 0;
    }
    // read potentiometers
    readPots();
    readSwitches();
#ifdef FRSKY_TELEMETRY 
    telemProc();
#endif    
    // digitalWrite(sigPin, (k&1)> 0?HIGH:LOW);
    
    // read keyboard value every 10th loop and store it
    // and set the values depending on the menu option
    displayReady=false;
    display.clearDisplay();
    if (pantalla==0) {
      // info screen
      unsigned char valBar;  
      display.setCursor(2,0);
      display.print("M: ");
      display.setCursor(15,0);
      display.print(modeloActual+1); 
      
#if 0 // trimmer keys keycode
      valBar = ~PCF8574_read8(PCF8574_TRIM_ADDR);
      display.setCursor(25,0);
      display.print(String(valBar));
#endif

#ifdef THROTLE_CUT
      // throtle cut
      if (bitRead(sw_status,0)==0) {
        display.setCursor(25, 0);
        display.print("TC");
      } 
      else {
        display.setCursor(25, 0);
        display.print("  ");
      }
#endif /* THROTLE_CUT */
      // dual rates
      display.setCursor(40, 0);
      display.print(bitRead(sw_status,1)==0?"R:LO":"R:HI");

      // batery Voltage
      display.setCursor(75, 0);
      display.print("Bat: ");  
      int voltajeBat=map(vBat,600,880,0,100);
      if(voltajeBat > 100) voltajeBat = 100;
      display.setCursor(98, 0);
      display.print(voltajeBat);
      display.setCursor(120, 0);
      display.print("%");


      // draw the bars
      display.drawLine(64,9,64,42, WHITE);
      for (i=0;i < 7;i++) {
        // level bars
        valBar=pots[i]/11,5;
        display.drawRect(15,9+(i*8),98,3, WHITE);
        display.drawPixel(16+valBar,10+(i*8), WHITE);
        display.drawPixel(15+valBar,10+(i*8), WHITE);
        display.drawPixel(17+valBar,10+(i*8), WHITE);
        display.setCursor(0, 8+(i*8));
        display.print(i+1); 
        // servo reverse indicator
        if (bitRead(servoReversa, i)) {
          display.setCursor(8, 8+(i*8));
          display.print("R"); 
        } 
        else {
          display.setCursor(8, 8+(i*8));
          display.print("N");      
        }
      }
    } 
    else {
      if (pantalla==1) {
        // main menu
        display.setCursor(0, 0);
        display.print(msgs[0]);
        for (i=1;i<=5;i++) {
          if (i==menuSubActual) {
            display.setCursor( 0, i*9);
            display.print("->");
            display.setCursor( 10, i*9);
            display.print(msgs[i]);
          } 
          else {
            display.setCursor( 0, i*9);
            display.print("  ");
            display.setCursor( 10, i*9);
            display.print(msgs[i]);
          }
        }
      } /* (pantalla==1) */

      if (pantalla==2) {
        if ( menuActual==1) {
          // servo reversing
          display.setCursor( 0, 0);
          display.print("(");  
          display.setCursor( 5, 0);
          display.print(modeloActual+1);
          display.setCursor( 10, 0);
          display.print(")");  
          display.setCursor( 20, 0);
          display.print(msgs[menuActual]);
          for (i=0;i < CANALES;i++) {
            if (i==menuSubActual) {
              display.setCursor( 0, 8+i*8);
              display.print("->");                   
            } 
            else {
              display.setCursor( 0, 8+i*8);
              display.print("  ");                
            }
            display.setCursor( 10, 8+i*8);
            display.print("CH: ");
            display.setCursor( 22, 8+i*8);
            display.print(i+1);
            if (bitRead(servoReversa,i)) {
              display.setCursor( 26, 8+i*8);
              display.print(" - R");
            } 
            else {
              display.setCursor(  26, 8+i*8);
              display.print(" - N");
            }
            display.setCursor( 55, 8+i*8);
            display.print(sub_trim[i]);
          }
        }

        if(menuActual==2) {
          // dual rates
          unsigned char contadorTemp=0;
          display.setCursor( 0, 0);
          display.print("(");  
          display.setCursor( 5, 0);
          display.print(modeloActual+1);
          display.setCursor( 10, 0);
          display.print(")");  
          display.setCursor( 20, 0);
          display.print(msgs[menuActual]);
          for (i=0;i < CANALES ;i++) {
            display.setCursor(0, 8+i*8);
            display.print("CH: ");
            display.setCursor(18, 8+i*8);
            display.print(i+1);
            // Lo Rate
            if (menuSubActual==contadorTemp) {
              if (drSeleccion==contadorTemp) {
                display.setCursor(26, 8+i*8);
                display.print("*");     
              } 
              else {
                display.setCursor(26, 8+i*8);
                display.print(">");
              }
            }
            display.setCursor(30, 8+i*8);
            display.print("L:" ); 
            display.setCursor(42, 8+i*8);
            display.print(String(dual_rate_low[i]));
            contadorTemp++; 

            // Hi Rate            
            if (menuSubActual==contadorTemp) {
              if (drSeleccion==contadorTemp) {
                display.setCursor( 55, 8+i*8);
                display.print("*");  
              } 
              else {
                display.setCursor( 55, 8+i*8);
                display.print(">");
              }
            }
            display.setCursor(  60, 8+i*8);
            display.print("H:");
            display.setCursor(  70, 8+i*8);
            display.print(String(dual_rate_hi[i]));
            contadorTemp++;

            // Exponent
            if (menuSubActual==contadorTemp) {
              if (drSeleccion==contadorTemp) {
                display.setCursor( 90, 8+i*8);
                display.print("*");  
              } 
              else {
                display.setCursor( 90, 8+i*8);
                display.print(">");
              }
            }
            display.setCursor(  95, 8+i*8);
            display.print("E:");
            display.setCursor(  105, 8+i*8);
            display.print(String(exponent[i]));
            contadorTemp++;


          }
        }

        if (menuActual==3) {
          // model selection
          display.setCursor(  0, 0);
          display.print(msgs[menuActual]);
          for (i=0;i<=(MODELOS-1);i++) {
            display.setCursor(  8, 8+i*8);
            display.print("Mod: ");
            display.setCursor(  27, 8+i*8);
            display.print(String(i+1) );
            if (i==menuSubActual) {
              display.setCursor(0, 8+i*8);
              display.print("->" );                   
            } 
            else {
              display.setCursor( 0, 8+i*8);
              display.print("  ");                
            }

            if (i==modeloActual) {
              display.setCursor(35, 8+i*8);
              display.print("*");
            }
          }
        }


        if (menuActual==4) {
          // data save
          saveModel2EEPROM(modeloActual);
          display.setCursor(  0, 20);
          display.print("Saving data for");
          display.setCursor(  0, 28);
          display.print("actual model:");
          display.setCursor(  0, 40);
          display.print(String(modeloActual+1));
          display.setCursor(  0, 42);
          display.print("...wait");
          pantalla--;
          menuActual=1;
          menuSubActual=1;
          display.display();
          delay(1000);
        }

        if (menuActual==5) {
          // gimbal calibration
          display.setCursor(  0, 0);
          display.print(msgs[menuActual]);
          display.display();
          delay(500);
          display.clearDisplay();
          for (i=0;i<=CANALES;i++) {
            calibracion[i][0]=512;
            calibracion[i][1]=512;
          }
          tecla=readKeys(); 
          while (tecla!=1) {
            display.clearDisplay();
            display.setCursor(  0, 0);
            display.print("Calibration");
            display.setCursor(  0, 8);
            display.print("Move the sitcks");
            for (i = 0; i < CANALES; i++) {
              if(inPots[i] == 255) continue;
              int lecturaTemporal=analogRead(inPots[i]);
              // min values
              if (lecturaTemporal<=calibracion[i][0]) {
                calibracion[i][0]=lecturaTemporal;
              }
              // max values
              if (lecturaTemporal>=calibracion[i][1]) {
                calibracion[i][1]=lecturaTemporal;
              }
              display.setCursor(  0, 16 + i*8);
              display.print(i+1);
              display.print(": ");
              display.setCursor(20, 16 + i*8);
              display.print(lecturaTemporal);
              display.setCursor(  40, 1616 + i*8);
              delay(10);
            }
            display.display();
            tecla=readKeys();
          }
          display.clearDisplay();
          display.setCursor(  0, 20);
          display.print("Saving calibration");
          display.setCursor(  0, 28);
          display.print("data");
          display.setCursor(   0, 40);
          display.print("...wait");
          display.display();
          // save calibration data
          int posEEPROM=0;
          for (i=0; i < CANALES;i++) {
            posEEPROM=CALIBR_EEPROM_OFFSET+(i*4);
            EEPROMWriteInt(posEEPROM, calibracion[i][0]); //min
            posEEPROM+=2;
            EEPROMWriteInt(posEEPROM, calibracion[i][1]); //max
          }
          delay(1000);
          pantalla=0;
          menuActual=0;
          menuSubActual=0;
        }
      }

    } /* (pantalla==2) */

#ifdef FRSKY_TELEMETRY    
    if((pantalla==2) && (menuActual == 0))
    {
       display.setCursor(0, 0);
       display.print("TX RSSI: "); 
       display.print(frsky.getLink_up());  
       display.setCursor(0, 10);
       display.print("Telemetry RSSI: "); 
       display.print(max(0, frsky.getLink_dn() - 100));  
       display.setCursor(0, 20);
       display.print("RX Voltage: "); 
       display.print(frsky.getRX_a1()*0.0517647058824);  // The internal sensor has a 4:1 divider, so the value is 0-13,2V in 255 steps or 0,052V per step
       display.setCursor(0, 30);
       display.print("A2 Voltage: "); 
       display.print(frsky.getRX_a2()*0.0129411764706);  // A2 without divider is 0-3,3V in 255 steps or 0,013V per step
       display.setCursor(0, 40);
       display.print("Rx Cnt: "); 
       display.print(frskyRxCnt);  // A2 without divider is 0-3,3V in 255 steps or 0,013V per stept
    } /* (pantalla==3) */
#endif
    // screen update
    display.display();


    // read keyboard every 2 loops
    if (j>2) {
      tecla=readKeys();
      readSubTrim(); 
      if (tecla==2) {
        if (pantalla<2) { 
          pantalla++; 
          menuActual=menuSubActual; 
          menuSubActual=0;
        } 
        else {
          // options for each menu
          if (menuActual==1) {
            // reversa de servos
            bitWrite(servoReversa, menuSubActual, !bitRead(servoReversa,menuSubActual));
          }
          if (menuActual==2) {
            // dual rates
            if (drSeleccion!=0xFF) {
              drSeleccion=0xFF;
            } 
            else {
              drSeleccion=menuSubActual;
            }
          }

          if (menuActual==3) {
            // model selection
            modeloActual=menuSubActual;
            // load data from EEPROM
            EEPROM.write(MODEL_EEPROM_OFFSET, modeloActual);
            cargaDatosEEPROM(modeloActual);
            pantalla=0; 
            menuSubActual=0; 
            menuActual=0;
          }
        }
      }

      if (tecla==3) {
        switch (menuActual) {
        case 2:
          if (drSeleccion==0xFF) {
            if (menuSubActual<(CANALES*3)) { 
              menuSubActual++; 
              if (pantalla==0) {
                pantalla++; 
              }
            }
          } 
          else {
            if ((drSeleccion%3)==0) {
              if (dual_rate_low[drSeleccion/3]<100) { 
                dual_rate_low[drSeleccion/3]++;  
              }
            } 
            else if ((drSeleccion%3)==1) {
              if (dual_rate_hi[drSeleccion/3]<100) { 
                dual_rate_hi[drSeleccion/3]++;  
              } 
            }
            else if ((drSeleccion%3)==2) {
              if (exponent[drSeleccion/3]<100) { 
                exponent[drSeleccion/3]++;  
              } 
            }
          }
          break;
        default: 
          if (menuSubActual<6) { 
            menuSubActual++; 
            if (pantalla==0) {
              pantalla++; 
            }
          }
          break;
        }   

      }

      if (tecla==1) {
        switch (menuActual) {
        case 2:
          if (drSeleccion==0xFF) {
            if (menuSubActual==0) { 
              pantalla--; 
              menuSubActual=1; 
              menuActual=0;
            } 
            else { 
              menuSubActual--; 
            } 
          } 
          else {
            if ((drSeleccion%3)==0) {
              if (dual_rate_low[drSeleccion/3]>0) { 
                dual_rate_low[drSeleccion/3]--;  
              }    
            } 
            else if ((drSeleccion%3) == 1) {
              if (dual_rate_hi[drSeleccion/3]>0) { 
                dual_rate_hi[drSeleccion/3]--;  
              } 
            }
            else if ((drSeleccion%3)== 2) {
              if (exponent[drSeleccion/3]>0) { 
                exponent[drSeleccion/3]--;  
              } 
            }
          }
          break;
        default:
          if (menuSubActual>0) {
            menuSubActual--; 
          } 
          else {
            if (pantalla>0) { 
              pantalla--; 
              menuSubActual=1; 
              menuActual=0;
            }
          } 
          break;
        }
      }

      j=0;
    }
    j++;

  }
}

#ifdef FRSKY_TELEMETRY 
void telemProc()
{
  while(frskTelemRx.available() > 0)
  {
    frskyRxCnt++;
    unsigned char c = frskTelemRx.read();
//    frsky.update(c);
  }
}
#endif

void readPots() {
  unsigned char i,j;
  for (i=0; i < CANALES; i++) 
  {
    // skip not analog channels
    if(inPots[i] == 255)
      continue;
    // read port & calibration mapping
    pots[i]=map(analogRead(inPots[i]),calibracion[i][0], calibracion[i][1], 0, 1023);

    // sub trim
    pots[i]= constrain(pots[i], 0, 1023);

    // servo reversing
    if (bitRead(servoReversa,i)==1) {
      pots[i]=1023-pots[i]; 
    }

#ifdef THROTLE_CUT 
    // Throtle cut
    if (i==canalTh && bitRead(sw_status,0)==0) {
      // TC activado
      pots[i]=0;
    }
#endif /* THROTLE_CUT */

    // dual rates
    if (bitRead(sw_status,1)==0) {
      // low, rates
      pots[i]=ajusteDR(pots[i],dual_rate_low[i],i,canalTh);
    } 
    else {
      // hi rates 
      pots[i]=ajusteDR(pots[i],dual_rate_hi[i],i,canalTh);
    }  
    pots[i] += sub_trim[i];
    // map values to PPM
    ppm[i]=map(pots[i],potMin, potMax, ppmMin, ppmMax);
    ppmOut.write(i+1, ppm[i]);
  }
}

void readSwitches() {
  // read switches and set into the sw_status var
#ifdef THROTLE_CUT 
  if (digitalRead(swTC)==HIGH) {
    bitWrite(sw_status,0,1);
  } 
  else {
    bitWrite(sw_status,0,0);
  }
#endif /* THROTLE_CUT */
  if (digitalRead(swDR)==HIGH) {
    bitWrite(sw_status,1,1);
  } 
  else {
    bitWrite(sw_status,1,0);
  }
}

void readSubTrim() {
  uint8_t st = ~PCF8574_read8(PCF8574_TRIM_ADDR);
  static uint8_t trimAcc = 1;
  if(st > 0)
  {
    int8_t chan, offs = 0;
    switch(st)
    {
      case 1:   chan = 0; offs = 1;  break;
      case 2:   chan = 0; offs = -1; break;
      case 4:   chan = 1; offs = 1;  break;
      case 8:   chan = 1; offs = -1; break;
      case 64:  chan = 3; offs = 1;  break;
      case 128: chan = 3; offs = -1; break;
      case 192: chan = 3; offs = -1; break;
    }  
    if(offs != 0)
    {
      sub_trim[chan] += ++trimAcc * offs;
      if((sub_trim[chan] > SUBTRIM_MAX) || (sub_trim[chan] < SUBTRIM_MIN))
      {
        sub_trim[chan] = constrain(sub_trim[chan], SUBTRIM_MIN, SUBTRIM_MAX);
          tone(tonePin, 800 , 200);
      } else
      {
        if(abs(sub_trim[chan]) < 5)
        { // sub trim centre
          tone(tonePin, 500 , 400);
          trimAcc = 1;
        }
        else
          tone(tonePin, 1000+ trimAcc*5 , 200);
        saveSubtrim = 20; // deley for auto save trims
      }
    }
  } 
  else
  {
    trimAcc = 4;
    if(saveSubtrim > 0)
    {
      if(--saveSubtrim == 1)
      { // save sub trims
        saveModel2EEPROM(modeloActual);
      }
    }    
  }
}

unsigned char  readKeys() {
  unsigned char salida;  
  salida = 0;
  if (digitalRead(4)==0) salida=3;
  if (digitalRead(2)==0) salida=2;
  if (digitalRead(3)==0) salida=1;
  delay(25); // !!!!!
  // reset auto save when key pressed
  if(salida != 0) saveSubtrim = 0; 
  return salida;
}

int  freeRam () {
  // returns free ram in avr chip
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}

void saveModel2EEPROM(unsigned char mod)
{
  int i;
  if(mod >= MODELOS) return; 
    int EEPROMPos =numBytesPorModelo*mod;
    // servo reverse
    EEPROM.write(EEPROMPos, servoReversa);
    EEPROMPos++;
    // dual rates
    for (i=0; i< CANALES; i++) {
      EEPROM.write(EEPROMPos, dual_rate_low[i]);
      EEPROMPos++;
      EEPROM.write(EEPROMPos, dual_rate_hi[i]);
      EEPROMPos++;
      EEPROM.write(EEPROMPos, exponent[i]);
      EEPROMPos++;
      EEPROMWriteInt(EEPROMPos, sub_trim[i]);
      EEPROMPos += 2;
    }
}

unsigned char cargaDatosEEPROM(unsigned char mod) {
  // load data from EEPROM, for selected model
  unsigned char i;
  if (mod >=  MODELOS) {
    mod=EEPROM.read(MODEL_EEPROM_OFFSET);
  }
  if(mod >= MODELOS) mod = 0;
  
  int EEPROMPos=numBytesPorModelo*mod;
  // servo reversing
  servoReversa=EEPROM.read(EEPROMPos);
  EEPROMPos++;
  // dual rates
  for (i=0;i < CANALES;i++) {
    dual_rate_low[i]=EEPROM.read(EEPROMPos);
    EEPROMPos++;
    dual_rate_hi[i]=EEPROM.read(EEPROMPos);
    EEPROMPos++;
    exponent[i]=EEPROM.read(EEPROMPos);
    EEPROMPos++;
    sub_trim[i] = EEPROMReadInt(EEPROMPos);
    EEPROMPos += 2;
  }
  // calibration 
  for (i=0;i < CANALES;i++) {
    EEPROMPos=CALIBR_EEPROM_OFFSET+(i*4);
    calibracion[i][0]=EEPROMReadInt(EEPROMPos); //minimo
    EEPROMPos+=2;
    calibracion[i][1]=EEPROMReadInt(EEPROMPos); //maximo
  }

  return mod;
}

unsigned char reseteaEEPROM() {
  // erases eeprom, just for checking values (unused now)
  int eepromBase, eepromPos;
  unsigned char i,j;
  EEPROM.write(MODEL_EEPROM_OFFSET,0);
  for (j=0;j < CANALES; j++) {
    eepromPos=numBytesPorModelo*j;
    EEPROM.write(eepromPos,0x00);
    eepromPos++;
    for (i=0; i < CANALES; i++) {
      if(i == canalTh)
        EEPROM.write(eepromPos,100);  // DR Lo
      else
        EEPROM.write(eepromPos,75);  // DR Lo
      eepromPos++;
      EEPROM.write(eepromPos,100); // DR Hi
      eepromPos++;
      EEPROM.write(eepromPos,0);   // Expo
      eepromPos++;
      EEPROMWriteInt(eepromPos,0);   // Subtr
      eepromPos += 2;
    }
  }      
}


int ajusteDR(int valor, float dr, unsigned char canal, unsigned char canTh) {
  // dual rate calc
  int salida,temp;
  float mult;
  mult=dr;
  if (canal==canalTh) {
    salida=(int)valor*mult/100;  
  } 
  else {
    temp=valor-512;
    salida=(int)((temp*mult/100)+512);

  }
  salida=constrain(salida,0,1023);
  return salida;
}


void EEPROMWriteInt(int p_address, int p_value) {
  // write a 16bit value in eeprom
  byte Byte1 = ((p_value >> 0) & 0xFF);
  byte Byte2 = ((p_value >> 8) & 0xFF);
  EEPROM.write(p_address, Byte1);
  EEPROM.write(p_address + 1, Byte2);
}

int EEPROMReadInt(int p_address) {
  // read a 16 bit value in eeprom
  byte Byte1 = EEPROM.read(p_address);
  byte Byte2 = EEPROM.read(p_address + 1);
  long firstTwoBytes = ((Byte1 << 0) & 0xFF) + ((Byte2 << 8) & 0xFF00);
  return (firstTwoBytes);
}

uint8_t PCF8574_read8(int address)
{
  uint8_t _data = 0;
  Wire.beginTransmission(address);
  Wire.requestFrom(address, 1);
  _data = Wire.read();
  Wire.endTransmission();
  return _data;
}





